#include "zx.h"
#include "zx_disp.h"
#include "zx_cbios.h"
#include "zx_driver.h"
#include "zx_version.h"

#define ZX_DEVICE_ATTR_RO(name) \
    static DEVICE_ATTR(name, 0444, zx_##name##_show, NULL)

#define ZX_DEVICE_ATTR_RW(name) \
    static DEVICE_ATTR(name, 0664, zx_##name##_show, zx_##name##_store)

#define ZX_DEVICE_ATTR_WO(name) \
    static DEVICE_ATTR(name, 0220, NULL, zx_##name##_store)

static ssize_t zx_engine_3d_usage_show(struct device *dev, struct device_attribute *attr, char *buf)
{
    int ret = 0;
    struct pci_dev*     pdev        = container_of(dev, struct pci_dev, dev);
    struct drm_device* drm_dev      = pci_get_drvdata(pdev);
    zx_card_t*         zx_card      = drm_dev->dev_private;

    zx_hwq_info hwq_info;
    ret = zx_core_interface->hwq_get_hwq_info(zx_card->adapter, &hwq_info);

    if (ret == 0)
    {
        return sprintf(buf, "%d\n", hwq_info.Usage_3D);
    }

    return sprintf(buf, "Not Enable!\n");
}

static ssize_t zx_engine_vcp_usage_show(struct device *dev, struct device_attribute *attr, char *buf)
{
    int ret = 0;
    struct pci_dev*     pdev        = container_of(dev, struct pci_dev, dev);
    struct drm_device* drm_dev      = pci_get_drvdata(pdev);
    zx_card_t*         zx_card      = drm_dev->dev_private;

    zx_hwq_info hwq_info;
    ret = zx_core_interface->hwq_get_hwq_info(zx_card->adapter, &hwq_info);

    if (ret == 0)
    {
        return sprintf(buf, "%d\n", hwq_info.Usage_VCP);
    }

    return sprintf(buf, "Not Enable!\n");
}

static ssize_t zx_engine_vpp_usage_show(struct device *dev, struct device_attribute *attr, char *buf)
{
    int ret = 0;
    struct pci_dev*     pdev        = container_of(dev, struct pci_dev, dev);
    struct drm_device* drm_dev      = pci_get_drvdata(pdev);
    zx_card_t*         zx_card      = drm_dev->dev_private;

    zx_hwq_info hwq_info;
    ret = zx_core_interface->hwq_get_hwq_info(zx_card->adapter, &hwq_info);

    if (ret == 0)
    {
        return sprintf(buf, "%d\n", hwq_info.Usage_VPP);
    }

    return sprintf(buf, "Not Enable!\n");
}

static ssize_t zx_mclk_show(struct device *dev, struct device_attribute *attr, char *buf)
{
    return sprintf(buf, "N/A\n");
}

static ssize_t zx_vclk_show(struct device *dev, struct device_attribute *attr, char *buf)
{
    return sprintf(buf, "N/A\n");
}

static ssize_t zx_eclk_show(struct device *dev, struct device_attribute *attr, char *buf)
{
    struct drm_device *drm_dev = dev_get_drvdata(dev);
    zx_card_t *zx_card = drm_dev->dev_private;
    disp_info_t *disp_info = (disp_info_t*)zx_card->disp_info;

    unsigned int value = 0;
    if(S_OK == disp_cbios_get_clock(disp_info, ZX_QUERY_ENGINE_CLOCK, &value))
    {
        return sprintf(buf, "%dMHz\n", (value + 5000)/10000);
    }

    return sprintf(buf, "Eclk Read Error\n");
}

static ssize_t zx_free_fb_mem_show(struct device *dev, struct device_attribute *attr, char *buf)
{
    return sprintf(buf, "N/A\n");
}

static ssize_t zx_fb_size_show(struct device *dev, struct device_attribute *attr, char *buf)
{
    struct drm_device *drm_dev = dev_get_drvdata(dev);
    zx_card_t *zx_card = drm_dev->dev_private;
    adapter_info_t *adapter_info = &zx_card->adapter_info;

    return sprintf(buf, "%d M\n", (adapter_info->fb_total_size >> 20));
}

static ssize_t zx_driver_version_show(struct device *dev, struct device_attribute *attr, char *buf)
{
    return sprintf(buf, "%02x.%02x.%02x%s\n",
                    DRIVER_MAJOR, DRIVER_MINOR, DRIVER_PATCHLEVEL, DRIVER_CLASS);
}

static ssize_t zx_release_date_show(struct device *dev, struct device_attribute *attr, char *buf)
{
    return sprintf(buf, "%s\n", DRIVER_DATE);
}

static ssize_t zx_vbios_version_show(struct device *dev, struct device_attribute *attr, char *buf)
{
    struct drm_device *drm_dev = dev_get_drvdata(dev);
    zx_card_t *zx_card = drm_dev->dev_private;
    disp_info_t *disp_info = (disp_info_t*)zx_card->disp_info;
    int vbiosVer = disp_info->vbios_version;

    return sprintf(buf, "%02x.%02x.%02x.%02x\n",
                    (vbiosVer>>24)&0xff, (vbiosVer>>16)&0xff, (vbiosVer>>8)&0xff, vbiosVer&0xff);
}

ZX_DEVICE_ATTR_RO(engine_3d_usage);
ZX_DEVICE_ATTR_RO(engine_vcp_usage);
ZX_DEVICE_ATTR_RO(engine_vpp_usage);
ZX_DEVICE_ATTR_RO(vbios_version);
ZX_DEVICE_ATTR_RO(release_date);
ZX_DEVICE_ATTR_RO(driver_version);
ZX_DEVICE_ATTR_RO(fb_size);
ZX_DEVICE_ATTR_RO(free_fb_mem);
ZX_DEVICE_ATTR_RO(eclk);
ZX_DEVICE_ATTR_RO(vclk);
ZX_DEVICE_ATTR_RO(mclk);

static struct attribute *zx_info_attributes[] = {
    &dev_attr_engine_3d_usage.attr,
    &dev_attr_engine_vpp_usage.attr,
    &dev_attr_engine_vcp_usage.attr,
    &dev_attr_vbios_version.attr,
    &dev_attr_release_date.attr,
    &dev_attr_driver_version.attr,
    &dev_attr_fb_size.attr,
    &dev_attr_free_fb_mem.attr,
    &dev_attr_eclk.attr,
    &dev_attr_vclk.attr,
    &dev_attr_mclk.attr,
    NULL
};

const struct attribute_group zx_sysfs_group = {
    .attrs = zx_info_attributes,
    .name  = "zx_info"
};


/**
 * Format such as:
 * VRAM total size:0x80000000
*/
static ssize_t zx_gpu_info_show(struct device *dev, struct device_attribute *attr, char *buf)
{
    ssize_t len = 0;
    unsigned int value = 0;
    struct drm_device *drm_dev = dev_get_drvdata(dev);
    zx_card_t *zx_card = drm_dev->dev_private;
    disp_info_t *disp_info = (disp_info_t*)zx_card->disp_info;
    adapter_info_t *adapter_info = &zx_card->adapter_info;

    len += sprintf(buf + len, "VRAM total size:0x%x\n", (adapter_info->fb_total_size));

    if(S_OK == disp_cbios_get_clock(disp_info, ZX_QUERY_ENGINE_CLOCK, &value))
    {
        len += sprintf(buf + len, "ECLK current:%u Mhz\n", (value + 5000)/10000);
    }
    else
    {
        len += sprintf(buf + len, "ECLK current:N/A Mhz\n");
    }

    return len;
}

static struct device_attribute dev_attr_gpu_info = __ATTR(gpu-info, 0444, zx_gpu_info_show, NULL);

const struct attribute *zx_os_gpu_info[] = {
    &dev_attr_gpu_info.attr,
    NULL
};